# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

class BillingRequestTemplate(object):
    """A thin wrapper around a billing_request_template, providing easy access to its
    attributes.

    Example:
      billing_request_template = client.billing_request_templates.get()
      billing_request_template.id
    """

    def __init__(self, attributes, api_response):
        self.attributes = attributes
        self.api_response = api_response

    @property
    def authorisation_url(self):
        return self.attributes.get('authorisation_url')
  

    @property
    def created_at(self):
        return self.attributes.get('created_at')
  

    @property
    def id(self):
        return self.attributes.get('id')
  

    @property
    def mandate_request_currency(self):
        return self.attributes.get('mandate_request_currency')
  

    @property
    def mandate_request_metadata(self):
        return self.attributes.get('mandate_request_metadata')
  

    @property
    def mandate_request_scheme(self):
        return self.attributes.get('mandate_request_scheme')
  

    @property
    def mandate_request_verify(self):
        return self.attributes.get('mandate_request_verify')
  

    @property
    def metadata(self):
        return self.attributes.get('metadata')
  

    @property
    def name(self):
        return self.attributes.get('name')
  

    @property
    def payment_request_amount(self):
        return self.attributes.get('payment_request_amount')
  

    @property
    def payment_request_currency(self):
        return self.attributes.get('payment_request_currency')
  

    @property
    def payment_request_description(self):
        return self.attributes.get('payment_request_description')
  

    @property
    def payment_request_metadata(self):
        return self.attributes.get('payment_request_metadata')
  

    @property
    def payment_request_scheme(self):
        return self.attributes.get('payment_request_scheme')
  

    @property
    def redirect_uri(self):
        return self.attributes.get('redirect_uri')
  

    @property
    def updated_at(self):
        return self.attributes.get('updated_at')
  


  

  

  

  

  

  

  

  

  

  

  

  

  

  

  

  

