# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

class BillingRequestFlow(object):
    """A thin wrapper around a billing_request_flow, providing easy access to its
    attributes.

    Example:
      billing_request_flow = client.billing_request_flows.get()
      billing_request_flow.id
    """

    def __init__(self, attributes, api_response):
        self.attributes = attributes
        self.api_response = api_response

    @property
    def authorisation_url(self):
        return self.attributes.get('authorisation_url')
  

    @property
    def auto_fulfil(self):
        return self.attributes.get('auto_fulfil')
  

    @property
    def created_at(self):
        return self.attributes.get('created_at')
  

    @property
    def exit_uri(self):
        return self.attributes.get('exit_uri')
  

    @property
    def expires_at(self):
        return self.attributes.get('expires_at')
  

    @property
    def id(self):
        return self.attributes.get('id')
  

    @property
    def links(self):
        return self.Links(self.attributes.get('links'))
  

    @property
    def lock_bank_account(self):
        return self.attributes.get('lock_bank_account')
  

    @property
    def lock_customer_details(self):
        return self.attributes.get('lock_customer_details')
  

    @property
    def redirect_uri(self):
        return self.attributes.get('redirect_uri')
  

    @property
    def session_token(self):
        return self.attributes.get('session_token')
  

    @property
    def show_redirect_buttons(self):
        return self.attributes.get('show_redirect_buttons')
  


  

  

  

  

  

  

  
    class Links(object):
        """Wrapper for the response's 'links' attribute."""

        def __init__(self, attributes):
            self.attributes = attributes
    
        @property
        def billing_request(self):
            return self.attributes.get('billing_request')
    
  

  

  

  

  

  

