# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

class Block(object):
    """A thin wrapper around a block, providing easy access to its
    attributes.

    Example:
      block = client.blocks.get()
      block.id
    """

    def __init__(self, attributes, api_response):
        self.attributes = attributes
        self.api_response = api_response

    @property
    def active(self):
        return self.attributes.get('active')
  

    @property
    def block_type(self):
        return self.attributes.get('block_type')
  

    @property
    def created_at(self):
        return self.attributes.get('created_at')
  

    @property
    def id(self):
        return self.attributes.get('id')
  

    @property
    def reason_description(self):
        return self.attributes.get('reason_description')
  

    @property
    def reason_type(self):
        return self.attributes.get('reason_type')
  

    @property
    def resource_reference(self):
        return self.attributes.get('resource_reference')
  

    @property
    def updated_at(self):
        return self.attributes.get('updated_at')
  


  

  

  

  

  

  

  

  

