# WARNING: Do not edit by hand, this file was generated by Crank:
#
#   https://github.com/gocardless/crank
#

class Institution(object):
    """A thin wrapper around a institution, providing easy access to its
    attributes.

    Example:
      institution = client.institutions.get()
      institution.id
    """

    def __init__(self, attributes, api_response):
        self.attributes = attributes
        self.api_response = api_response

    @property
    def country_code(self):
        return self.attributes.get('country_code')
  

    @property
    def icon_url(self):
        return self.attributes.get('icon_url')
  

    @property
    def id(self):
        return self.attributes.get('id')
  

    @property
    def logo_url(self):
        return self.attributes.get('logo_url')
  

    @property
    def name(self):
        return self.attributes.get('name')
  


  

  

  

  

  

